<?php

namespace Botble\TifrFacultyRecruitmentApplication\Tables;

use Botble\Table\Abstracts\TableAbstract;
use Botble\Table\Actions\EditAction;
use Botble\Table\Columns\CreatedAtColumn;
use Botble\Table\Columns\IdColumn;
use Botble\Table\Columns\NameColumn;
use Botble\Table\Columns\Column;
use Illuminate\Database\Eloquent\Builder;

class TifrFacultyRecruitmentApplicationTable extends TableAbstract
{
    public function setup(): void
    {
        $this
            ->addActions([
                EditAction::make()->route('tifrfacultyrecruitmentapplication.edit'),
                $this->viewEvaluationAction(),
            ])
            ->addColumns([
                IdColumn::make(),
                NameColumn::make('full_name')->label('Full Name')->route('tifrfacultyrecruitmentapplication.edit'),
                Column::make('gender')->label('Gender'),
                Column::make('email')->label('Email'),
                Column::make('phone')->label('Phone'),
                Column::make('evaluation_status')->label('Evaluation'),
                Column::make('evaluation_details')->label('Evaluation Details'),
                CreatedAtColumn::make(),
            ])
            ->queryUsing(function (Builder $query) {
                $this->applyRoleBasedQuery($query);
            });
    }

    // Simplified action generation for view button
    private function viewEvaluationAction()
    {
        return \Botble\Table\Actions\Action::make('view')
            ->label('View Evaluations')
            ->color('primary')
            ->route('tifrfacultyrecruitmentapplication.committee.recommendations', ['id' => '{id}'])
            ->attributes(['target' => '_blank'])
            ->icon('ti ti-eye');
    }

    // Role-based query applied in setup
    private function applyRoleBasedQuery(Builder $query)
    {
        $userRole = auth()->user()->roles()->first()->name;

        if ($userRole == 'Committee Member' || $userRole == 'Chairperson') {
            $query->from('tifr_faculty_registrations')
                ->leftJoin('tifr_faculty_registration_evaluation_statuses', 'tifr_faculty_registrations.id', '=', 'tifr_faculty_registration_evaluation_statuses.faculty_recruitment_application_id')
                ->leftJoin('tifr_faculty_registration_evaluations', 'tifr_faculty_registrations.id', '=', 'tifr_faculty_registration_evaluations.faculty_recruitment_application_id')
                ->select([
                    'tifr_faculty_registrations.id',
                    \DB::raw('CONCAT(tifr_faculty_registrations.first_name, " ", IFNULL(tifr_faculty_registrations.middle_name, ""), " ", tifr_faculty_registrations.last_name) AS full_name'),
                    'tifr_faculty_registrations.gender',
                    'tifr_faculty_registrations.email',
                    'tifr_faculty_registrations.phone',
                    \DB::raw('
                        CASE 
                            WHEN tifr_faculty_registration_evaluation_statuses.is_complete IS NULL OR tifr_faculty_registration_evaluation_statuses.is_complete != "Yes" THEN "Pending"
                            ELSE "Done"
                        END AS evaluation_status
                    '),
                    'tifr_faculty_registration_evaluation_statuses.is_complete',
                    'tifr_faculty_registrations.created_at',
                    \DB::raw('COALESCE(CONCAT(
                        "Grade - ", tifr_faculty_registration_evaluations.grade, "<br>",
                        "Recommendation - ", tifr_faculty_registration_evaluations.recommendation, "<br>",
                        "Comment - ", 
                        IFNULL(
                            CONCAT(
                                "<i class=\"fa-solid fa-circle-info\" style=\"cursor: pointer; color: #007bff; font-weight: bold; font-size: 16px;\" title=\"", tifr_faculty_registration_evaluations.comments, "\"></i>"),
                            "-")
                    ), "-") AS evaluation_details')
                ]);
        } else {
            // No data is returned for other roles
            $query->from('tifr_faculty_registrations')
                ->whereRaw('1 = 0');
        }
    }

    // Optimized getRowActions method
    public function getRowActions(): array
    {
        $actions = parent::getRowActions();
        $userRole = auth()->user()->roles()->first()->name;

        // Only apply the condition to Committee Members or Chairpersons
        if ($userRole == 'Committee Member' || $userRole == 'Chairperson') {
            $actions = $this->filterViewActionByEvaluationStatus($actions);
        } else {
            $actions = [];  // Remove all actions for other roles
        }

        return $actions;
    }

    // Filter 'view' action based on evaluation status
    private function filterViewActionByEvaluationStatus(array $actions): array
    {
        // Here we are using the query from the setup method to check the status
        $data = \DB::table('tifr_faculty_registrations')
            ->leftJoin('tifr_faculty_registration_evaluation_statuses', 'tifr_faculty_registrations.id', '=', 'tifr_faculty_registration_evaluation_statuses.faculty_recruitment_application_id')
            ->leftJoin('tifr_faculty_registration_evaluations', 'tifr_faculty_registrations.id', '=', 'tifr_faculty_registration_evaluations.faculty_recruitment_application_id')
            ->select([
                'tifr_faculty_registrations.id',
                \DB::raw('CONCAT(tifr_faculty_registrations.first_name, " ", IFNULL(tifr_faculty_registrations.middle_name, ""), " ", tifr_faculty_registrations.last_name) AS full_name'),
                'tifr_faculty_registration_evaluation_statuses.is_complete'
            ])
            ->get();

        // Check if any of the rows have evaluation status as 'Done'
        foreach ($data as $row) {
            if ($row->is_complete != 'Yes') {
                // Remove the 'view' action if evaluation is not 'Done'
                foreach ($actions as $key => $action) {
                    if ($key === 'view') {
                        unset($actions[$key]);
                    }
                }
            }
        }

        return $actions;
    }
}
